@@ -816,10 +816,17 @@ char *query_crsnic(const int sock, const char *query)
     int hide = hide_discl;
     char *referral_server = NULL;
     int state = 0;
+    int dotscount = 0;
 
     temp = malloc(strlen("domain ") + strlen(query) + 2 + 1);
+    *temp = '\0';
 
-    if (!strpbrk(query, "=~ "))
+    /* if this has more than one dot then it is a name server */
+    for (p = (char *) query; *p != '\0'; p++)
+	if (*p == '.')
+	    dotscount++;
+
+    if (dotscount == 1 && !strpbrk(query, "=~ "))
 	strcpy(temp, "domain ");
     strcat(temp, query);
     strcat(temp, "\r\n");
@@ -834,8 +841,11 @@ char *query_crsnic(const int sock, const char *query)
 	   is queried */
 	if (state == 0 && strneq(buf, "   Domain Name:", 15))
 	    state = 1;
-	if (state == 1 && (strneq(buf, "   Whois Server:", 16)
-		    || strneq(buf, "   WHOIS Server:", 16))) {
+	if (state == 0 && strneq(buf, "   Server Name:", 15)) {
+	    referral_server = strdup("");
+	    state = 2;
+	}
+	if (state == 1 && strneq(buf, "   Registrar WHOIS Server:", 26)) {
 	    for (p = buf; *p != ':'; p++);	/* skip until the colon */
 	    for (p++; *p == ' '; p++);		/* skip the spaces */
 	    referral_server = strdup(p);
@@ -884,7 +894,7 @@ char *query_afilias(const int sock, const char *query)
 	   This is not supposed to happen. */
 	if (state == 0 && strneq(buf, "Domain Name:", 12))
 	    state = 1;
-	if (state == 1 && strneq(buf, "Whois Server:", 13)) {
+	if (state == 1 && strneq(buf, "Registrar WHOIS Server:", 23)) {
 	    for (p = buf; *p != ':'; p++);	/* skip until colon */
 	    for (p++; *p == ' '; p++);		/* skip colon and spaces */
 	    referral_server = strdup(p);
